@@ -10,10 +10,7 @@
 
 # get a list of entries for a given code list dictionary
 
-from __future__ import absolute_import
-from __future__ import print_function
 import sys
-import urllib2
 
 from owslib.etree import etree
 from owslib.iso import CodelistCatalogue
