@@ -20,6 +20,8 @@ from owslib import fes
 from owslib import util
 from owslib import ows
 from owslib.iso import MD_Metadata, FC_FeatureCatalogue
+from owslib.iso3 import MD_Metadata as MD_Metadata3  # ISO 19115 Part 3 XML
+from owslib.iso3 import FC_FeatureCatalogue as FC_FeatureCatalogue3  # ISO 19115 Part 3 XML
 from owslib.fgdc import Metadata
 from owslib.dif import DIF
 from owslib.gm03 import GM03
@@ -43,7 +45,7 @@ schema_location = '%s %s' % (namespaces['csw'], schema)
 class CatalogueServiceWeb(object):
     """ csw request class """
     def __init__(self, url, lang='en-US', version='2.0.2', timeout=10, skip_caps=False,
-                 username=None, password=None, auth=None):
+                 username=None, password=None, auth=None, headers=None):
         """
 
         Construct and process a GetCapabilities request
@@ -59,6 +61,7 @@ class CatalogueServiceWeb(object):
         - username: username for HTTP basic authentication
         - password: password for HTTP basic authentication
         - auth: instance of owslib.util.Authentication
+        - headers: HTTP headers to send with requests
 
         """
         if auth:
@@ -71,6 +74,7 @@ class CatalogueServiceWeb(object):
         self.version = version
         self.timeout = timeout
         self.auth = auth or Authentication(username, password)
+        self.headers = headers
         self.service = 'CSW'
         self.exceptionreport = None
         self.owscommon = ows.OwsCommon('1.0.0')
@@ -168,22 +172,25 @@ class CatalogueServiceWeb(object):
 
         self.request = node0
 
-        self._invoke()
+        try:
+            self._invoke()
 
-        if self.exceptionreport is None:
-            self.results = {}
+            if self.exceptionreport is None:
+                self.results = {}
 
-            val = self._exml.find(util.nspath_eval('csw:DomainValues', namespaces)).attrib.get('type')
-            self.results['type'] = util.testXMLValue(val, True)
+                val = self._exml.find(util.nspath_eval('csw:DomainValues', namespaces)).attrib.get('type')
+                self.results['type'] = util.testXMLValue(val, True)
 
-            val = self._exml.find(util.nspath_eval('csw:DomainValues/csw:%s' % dtypename, namespaces))
-            self.results[dtype] = util.testXMLValue(val)
+                val = self._exml.find(util.nspath_eval('csw:DomainValues/csw:%s' % dtypename, namespaces))
+                self.results[dtype] = util.testXMLValue(val)
 
-            # get the list of values associated with the Domain
-            self.results['values'] = []
+                # get the list of values associated with the Domain
+                self.results['values'] = []
 
-            for f in self._exml.findall(util.nspath_eval('csw:DomainValues/csw:ListOfValues/csw:Value', namespaces)):
-                self.results['values'].append(util.testXMLValue(f))
+                for f in self._exml.findall(util.nspath_eval('csw:DomainValues/csw:ListOfValues/csw:Value', namespaces)): # noqa
+                    self.results['values'].append(util.testXMLValue(f))
+        except Exception:
+            self.results = {'values': []}
 
     def getrecords(self, qtype=None, keywords=[], typenames='csw:Record', propertyname='csw:AnyText', bbox=None,
                    esn='summary', sortby=None, outputschema=namespaces['csw'], format=outputformat, startposition=0,
@@ -224,11 +231,6 @@ class CatalogueServiceWeb(object):
         else:
             # construct request
             node0 = self._setrootelement('csw:GetRecords')
-            if etree.__name__ != 'lxml.etree':  # apply nsmap manually
-                node0.set('xmlns:ows', namespaces['ows'])
-                node0.set('xmlns:gmd', namespaces['gmd'])
-                node0.set('xmlns:dif', namespaces['dif'])
-                node0.set('xmlns:fgdc', namespaces['fgdc'])
             node0.set('outputSchema', outputschema)
             node0.set('outputFormat', format)
             node0.set('version', self.version)
@@ -334,7 +336,12 @@ class CatalogueServiceWeb(object):
         """
 
         if xml is not None:
-            if xml.startswith(b'<'):
+            if isinstance(xml, bytes):
+                startswith_xml = xml.startswith(b'<')
+            else:  # str
+                startswith_xml = xml.startswith('<')
+
+            if startswith_xml:
                 self.request = etree.fromstring(xml)
                 val = self.request.find(util.nspath_eval('csw:Query/csw:ElementSetName', namespaces))
                 if val is not None:
@@ -347,11 +354,6 @@ class CatalogueServiceWeb(object):
         else:
             # construct request
             node0 = self._setrootelement('csw:GetRecords')
-            if etree.__name__ != 'lxml.etree':  # apply nsmap manually
-                node0.set('xmlns:ows', namespaces['ows'])
-                node0.set('xmlns:gmd', namespaces['gmd'])
-                node0.set('xmlns:dif', namespaces['dif'])
-                node0.set('xmlns:fgdc', namespaces['fgdc'])
             node0.set('outputSchema', outputschema)
             node0.set('outputFormat', format)
             node0.set('version', self.version)
@@ -581,6 +583,12 @@ class CatalogueServiceWeb(object):
                 val = i.find(util.nspath_eval('gm03:fileIdentifier', namespaces))
                 identifier = self._setidentifierkey(util.testXMLValue(val))
                 self.records[identifier] = GM03(i)
+        elif MD_Metadata3.handles(outputschema):  # ISO 19115 Part 3 XML
+            for elem, id in MD_Metadata3.find_ids(self._exml):
+                self.records[self._setidentifierkey(id)] = MD_Metadata3(elem)
+            for i in self._exml.findall('.//' + util.nspath_eval('gfc:FC_FeatureCatalogue', namespaces)):
+                identifier = self._setidentifierkey(util.testXMLValue(i.attrib['uuid'], attrib=True))
+                self.records[identifier] = FC_FeatureCatalogue3(i)
         else:  # process default
             for i in self._exml.findall('.//' + util.nspath_eval('csw:%s' % self._setesnel(esn), namespaces)):
                 val = i.find(util.nspath_eval('dc:identifier', namespaces))
@@ -615,10 +623,7 @@ class CatalogueServiceWeb(object):
             return el
 
     def _setrootelement(self, el):
-        if etree.__name__ == 'lxml.etree':  # apply nsmap
-            return etree.Element(util.nspath_eval(el, namespaces), nsmap=namespaces)
-        else:
-            return etree.Element(util.nspath_eval(el, namespaces))
+        return etree.Element(util.nspath_eval(el, namespaces), nsmap=namespaces)
 
     def _setconstraint(self, parent, qtype=None, propertyname='csw:AnyText', keywords=[], bbox=None, cql=None,
                        identifier=None):
@@ -659,15 +664,16 @@ class CatalogueServiceWeb(object):
                     post_verbs = [x for x in op.methods if x.get('type').lower() == 'post']
                     if len(post_verbs) > 1:
                         # Filter by constraints.  We must match a PostEncoding of "XML"
+                        found_xml = False
                         for pv in post_verbs:
                             for const in pv.get('constraints'):
                                 if const.name.lower() == 'postencoding':
                                     values = [v.lower() for v in const.values]
                                     if 'xml' in values:
                                         request_url = pv.get('url')
+                                        found_xml = True
                                         break
-                        else:
-                            # Well, just use the first one.
+                        if not found_xml:  # Well, just use the first one.
                             request_url = post_verbs[0].get('url')
                     elif len(post_verbs) == 1:
                         request_url = post_verbs[0].get('url')
@@ -677,8 +683,8 @@ class CatalogueServiceWeb(object):
         if isinstance(self.request, str):  # GET KVP
             self.request = '%s%s' % (bind_url(request_url), self.request)
             self.response = openURL(
-                self.request, None, 'Get', timeout=self.timeout, auth=self.auth
-            ).read()
+                self.request, None, 'Get', timeout=self.timeout, auth=self.auth,
+                headers=self.headers).read()
         else:
             self.request = cleanup_namespaces(self.request)
             # Add any namespaces used in the "typeNames" attribute of the
@@ -694,7 +700,8 @@ class CatalogueServiceWeb(object):
 
             self.request = util.element_to_string(self.request, encoding='utf-8')
 
-            self.response = http_post(request_url, self.request, self.lang, self.timeout, auth=self.auth)
+            self.response = http_post(request_url, self.request, self.lang, self.timeout,
+                                      auth=self.auth, headers=self.headers).content
 
         # parse result see if it's XML
         self._exml = etree.parse(BytesIO(self.response))
@@ -712,7 +719,7 @@ class CatalogueServiceWeb(object):
         ]
 
         if self._exml.getroot().tag not in valid_xpaths:
-            raise RuntimeError('Document is XML, but not CSW-ish')
+            raise RuntimeError(f'Document is XML, but not CSW-ish, {request_url}?{self.request}')
 
         # check if it's an OGC Exception
         val = self._exml.find(util.nspath_eval('ows:Exception', namespaces))
