@@ -13,7 +13,7 @@ import sys
 from urllib.parse import urlencode, parse_qsl
 from owslib.etree import etree
 from owslib.namespaces import Namespaces
-from owslib.util import which_etree, findall, Authentication, openURL
+from owslib.util import findall, Authentication, openURL
 
 MYNS = Namespaces()
 XS_NAMESPACE = MYNS.get_namespace("xs")
@@ -52,6 +52,8 @@ def get_schema(
     if ":" in typename:
         typename = typename.split(":")[1]
     type_element = root.find("./{%s}element" % XS_NAMESPACE)
+    if type_element is None:
+        return None
     complex_type = type_element.attrib["type"].split(":")[1]
     elements = _get_elements(complex_type, root)
     nsmap = None
@@ -84,7 +86,8 @@ def _construct_schema(elements, nsmap):
 
     :return dict: schema
     """
-
+    if elements is None:
+        return None
     schema = {"properties": {}, "required": [], "geometry": None}
 
     schema_key = None
@@ -93,13 +96,17 @@ def _construct_schema(elements, nsmap):
     # if nsmap is defined, use it
     if nsmap:
         for key in nsmap:
-            if nsmap[key] == XS_NAMESPACE:
-                schema_key = key
-            if nsmap[key] in GML_NAMESPACES:
-                gml_key = key
-    # if no nsmap is defined, we have to guess
-    else:
+            if key is not None:
+                if nsmap[key] == XS_NAMESPACE:
+                    schema_key = key
+                if nsmap[key] in GML_NAMESPACES:
+                    gml_key = key
+
+    # if no nsmap keys are defined, we have to guess
+    if gml_key is None:
         gml_key = "gml"
+
+    if schema_key is None:
         schema_key = "xsd"
 
     mappings = {
@@ -124,7 +131,8 @@ def _construct_schema(elements, nsmap):
             schema["geometry"] = mappings[data_type]
             schema["geometry_column"] = name
         else:
-            schema["properties"][name] = data_type.replace(schema_key + ":", "")
+            if schema_key is not None:
+                schema["properties"][name] = data_type.replace(schema_key + ":", "")
 
         if non_nillable:
             schema["required"].append(name)
