@@ -6,10 +6,15 @@
 # $Id: wfs.py 503 2006-02-01 17:09:12Z dokai $
 # =============================================================================
 
+from io import BytesIO
+import logging
+from urllib.parse import urlencode
+
 # owslib imports:
 from owslib import util
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
+from owslib.iso3 import MD_Metadata as MD_Metadata3  # ISO 19115 Part 3 XML
 from owslib.ows import Constraint, ServiceIdentification, ServiceProvider, OperationsMetadata
 from owslib.etree import etree
 from owslib.util import nspath, testXMLValue, openURL, Authentication
@@ -17,16 +22,11 @@ from owslib.crs import Crs
 from owslib.feature import WebFeatureService_
 from owslib.feature.common import (
     WFSCapabilitiesReader,
-    AbstractContentMetadata,
-    makeStringIO,
+    AbstractContentMetadata
 )
 from owslib.namespaces import Namespaces
 
-# other imports
-from urllib.parse import urlencode
-
-import logging
-from owslib.util import log
+LOGGER = logging.getLogger(__name__)
 
 n = Namespaces()
 WFS_NAMESPACE = n.get_namespace("wfs20")
@@ -45,48 +45,6 @@ class WebFeatureService_2_0_0(WebFeatureService_):
 
     Implements IWebFeatureService.
     """
-
-    def __new__(
-        self,
-        url,
-        version,
-        xml,
-        parse_remote_metadata=False,
-        timeout=30,
-        headers=None,
-        username=None,
-        password=None,
-        auth=None,
-    ):
-        """ overridden __new__ method
-
-        @type url: string
-        @param url: url of WFS capabilities document
-        @type xml: string
-        @param xml: elementtree object
-        @type parse_remote_metadata: boolean
-        @param parse_remote_metadata: whether to fully process MetadataURL elements
-        @param headers: HTTP headers to send with requests
-        @param timeout: time (in seconds) after which requests should timeout
-        @param username: service authentication username
-        @param password: service authentication password
-        @param auth: instance of owslib.util.Authentication
-        @return: initialized WebFeatureService_2_0_0 object
-        """
-        obj = object.__new__(self)
-        obj.__init__(
-            url,
-            version,
-            xml,
-            parse_remote_metadata,
-            timeout,
-            headers=headers,
-            username=username,
-            password=password,
-            auth=auth,
-        )
-        return obj
-
     def __getitem__(self, name):
         """ check contents dictionary to allow dict like access to service layers"""
         if name in list(self.__getattribute__("contents").keys()):
@@ -115,7 +73,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         else:
             auth = Authentication()
         super(WebFeatureService_2_0_0, self).__init__(auth)
-        log.debug("building WFS %s" % url)
+        LOGGER.debug("building WFS %s" % url)
         self.url = url
         self.version = version
         self.timeout = timeout
@@ -125,7 +83,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         if xml:
             self._capabilities = reader.readString(xml)
         else:
-            self._capabilities = reader.read(self.url)
+            self._capabilities = reader.read(self.url, self.timeout)
         self._buildMetadata(parse_remote_metadata)
 
     def _buildMetadata(self, parse_remote_metadata=False):
@@ -141,16 +99,17 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         featuretypelistelem = self._capabilities.find(
             nspath("FeatureTypeList", ns=WFS_NAMESPACE)
         )
-        featuretypeelems = featuretypelistelem.findall(
-            nspath("FeatureType", ns=WFS_NAMESPACE)
-        )
-        if serviceidentelem is not None:
-            for f in featuretypeelems:
-                kwds = f.findall(nspath("Keywords/Keyword", ns=OWS_NAMESPACE))
-                if kwds is not None:
-                    for kwd in kwds[:]:
-                        if kwd.text not in self.identification.keywords:
-                            self.identification.keywords.append(kwd.text)
+        if featuretypelistelem is not None:
+            featuretypeelems = featuretypelistelem.findall(
+                nspath("FeatureType", ns=WFS_NAMESPACE)
+            )
+            if serviceidentelem is not None:
+                for f in featuretypeelems:
+                    kwds = f.findall(nspath("Keywords/Keyword", ns=OWS_NAMESPACE))
+                    if kwds is not None:
+                        for kwd in kwds[:]:
+                            if kwd.text not in self.identification.keywords:
+                                self.identification.keywords.append(kwd.text)
 
         # TODO: update serviceProvider metadata, miss it out for now
         serviceproviderelem = self._capabilities.find(nspath("ServiceProvider"))
@@ -225,6 +184,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         featureversion=None,
         propertyname=None,
         maxfeatures=None,
+        srsname=None,
         storedQueryID=None,
         storedQueryParams=None,
         method="Get",
@@ -233,7 +193,9 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         sortby=None,
     ):
         """Request and return feature data as a file-like object.
+
         #TODO: NOTE: have changed property name from ['*'] to None - check the use of this in WFS 2.0
+
         Parameters
         ----------
         typename : list
@@ -247,9 +209,18 @@ class WebFeatureService_2_0_0(WebFeatureService_):
         featureversion : string
             Default is most recent feature version.
         propertyname : list
-            List of feature property names. '*' matches all.
+            List of feature property names. For Get request, '*' matches all.
+            For Post request, leave blank (None) to get all properties.
         maxfeatures : int
             Maximum number of features to be returned.
+        srsname: string
+            EPSG code to request the data in
+        storedQueryID : string
+            A name identifying a prepared set available in WFS-service
+        storedQueryParams : dict
+            Variable amount of extra information sent to server related to
+            storedQueryID to further define the requested data
+            {'parameter_name': parameter_value}
         method : string
             Qualified name of the HTTP DCP method to use.
         outputFormat: string (optional)
@@ -261,11 +232,19 @@ class WebFeatureService_2_0_0(WebFeatureService_):
             (upon presentation) the set of feature instances that
             satify the query.
 
-        There are 3 different modes of use
+        There are 5 different modes of use
 
         1) typename and bbox (simple spatial query)
         2) typename and filter (==query) (more expressive)
         3) featureid (direct access to known features)
+        4) storedQueryID and optional storedQueryParams
+        5) filter only via Post method
+
+        Raises:
+            ServiceException: If there is an error during the request
+
+        Returns:
+            BytesIO -- Data returned from the service as a file-like object
         """
         storedQueryParams = storedQueryParams or {}
         url = data = None
@@ -280,6 +259,7 @@ class WebFeatureService_2_0_0(WebFeatureService_):
                 featureversion,
                 propertyname,
                 maxfeatures,
+                srsname,
                 storedQueryID,
                 storedQueryParams,
                 outputFormat,
@@ -287,11 +267,23 @@ class WebFeatureService_2_0_0(WebFeatureService_):
                 startindex,
                 sortby,
             )
-            log.debug("GetFeature WFS GET url %s" % url)
+            LOGGER.debug("GetFeature WFS GET url %s" % url)
         else:
-            (url, data) = self.getPOSTGetFeatureRequest()
+            url, data = self.getPOSTGetFeatureRequest(
+                typename,
+                filter,
+                bbox,
+                featureid,
+                featureversion,
+                propertyname,
+                maxfeatures,
+                storedQueryID,
+                storedQueryParams,
+                outputFormat,
+                "Post",
+                startindex,
+                sortby)
 
-        # If method is 'Post', data will be None here
         u = openURL(url, data, method, timeout=self.timeout, headers=self.headers, auth=self.auth)
 
         # check for service exceptions, rewrap, and return
@@ -313,16 +305,16 @@ class WebFeatureService_2_0_0(WebFeatureService_):
                 tree = etree.fromstring(data)
             except BaseException:
                 # Not XML
-                return makeStringIO(data)
+                return BytesIO(data)
             else:
                 if tree.tag == "{%s}ServiceExceptionReport" % OGC_NAMESPACE:
                     se = tree.find(nspath("ServiceException", OGC_NAMESPACE))
                     raise ServiceException(str(se.text).strip())
                 else:
-                    return makeStringIO(data)
+                    return BytesIO(data)
         else:
             if have_read:
-                return makeStringIO(data)
+                return BytesIO(data)
             return u
 
     def getpropertyvalue(
@@ -499,7 +491,7 @@ class ContentMetadata(AbstractContentMetadata):
         else:
             self.abstract = None
         self.keywords = [
-            f.text for f in elem.findall(nspath("Keywords", ns=WFS_NAMESPACE))
+            f.text for f in elem.findall(nspath("Keywords/Keyword", ns=OWS_NAMESPACE))
         ]
 
         # bboxes
@@ -581,5 +573,12 @@ class ContentMetadata(AbstractContentMetadata):
                     if mdelem is not None:
                         metadataUrl["metadata"] = MD_Metadata(mdelem)
                         continue
+                    else:  # ISO 19115 Part 3 XML
+                        mdelem = MD_Metadata3.find_start(doc)
+                        if mdelem is not None:
+                            metadataUrl["metadata"] = MD_Metadata3(mdelem)
+                        else:
+                            metadataUrl["metadata"] = None
+                        continue
                 except Exception:
                     metadataUrl["metadata"] = None
