@@ -23,12 +23,11 @@ from owslib.util import (openURL, ServiceException, testXMLValue,
                          nspath_eval, bind_url, Authentication)
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
+from owslib.iso3 import MD_Metadata as MD_Metadata3  # ISO 19115 Part 3 XML
 from owslib.crs import Crs
 from owslib.namespaces import Namespaces
 from owslib.map.common import WMSCapabilitiesReader, AbstractContentMetadata
 
-from owslib.util import log
-
 n = Namespaces()
 WMS_NAMESPACE = n.get_namespace("wms")
 OGC_NAMESPACE = n.get_namespace('ogc')
@@ -69,7 +68,7 @@ class WebMapService_1_3_0(object):
         # Authentication handled by Reader
         reader = WMSCapabilitiesReader(
             self.version, url=self.url, headers=headers, auth=self.auth)
-        if xml:  # read from stored xml
+        if xml is not None:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
             self._capabilities = reader.read(self.url, timeout=self.timeout)
@@ -183,12 +182,14 @@ class WebMapService_1_3_0(object):
 
         # remapping the srs to crs for the request
         request['crs'] = str(srs)
-        request['bbox'] = ','.join([repr(x) for x in bbox])
+        request['bbox'] = ','.join([str(x) for x in bbox])
         request['format'] = str(format)
         request['transparent'] = str(transparent).upper()
-        request['bgcolor'] = '0x' + bgcolor[1:7]
         request['exceptions'] = str(exceptions)
 
+        if bgcolor:
+            request['bgcolor'] = '0x' + bgcolor[1:7]
+
         if time is not None:
             request['time'] = str(time)
 
@@ -305,7 +306,7 @@ class WebMapService_1_3_0(object):
 
         self.request = bind_url(base_url) + data
 
-        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth)
+        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth, headers=self.headers)
 
         # need to handle casing in the header keys
         headers = {}
@@ -380,7 +381,7 @@ class WebMapService_1_3_0(object):
 
         self.request = bind_url(base_url) + data
 
-        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth)
+        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth, headers=self.headers)
 
         # check for service exceptions, and return
         if u.info()['Content-Type'] == 'XML':
@@ -644,7 +645,9 @@ class ContentMetadata(AbstractContentMetadata):
                 if k != 'name':
                     dim_data[k] = v
             # single values and ranges are not differentiated here
-            dim_data['values'] = dim.text.strip().split(',') if dim.text.strip() else None
+            dim_data['values'] = None
+            if dim.text:
+                dim_data['values'] = dim.text.strip().split(',') if dim.text.strip() else None
             self.dimensions[dim_name] = dim_data
 
         # MetadataURLs
@@ -705,6 +708,12 @@ class ContentMetadata(AbstractContentMetadata):
                     if mdelem is not None:
                         metadataUrl['metadata'] = MD_Metadata(mdelem)
                         continue
+                    else:
+                        mdelem = MD_Metadata3.find_start(doc)
+                        if mdelem is not None:
+                            metadataUrl["metadata"] = MD_Metadata3(mdelem)
+                        else:
+                            metadataUrl["metadata"] = None
                 except Exception:
                     metadataUrl['metadata'] = None
 
